package info.textgrid.middleware;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.search.SearchHit;

/**
 * <p>
 * The Dublin Core Field Loader takes the results of the 
 * ElasticSearch request and puts them into a String-List. 
 * For each element of Dublin Core exists a seperate list. 
 * 
 * For each element two functions are necessary because the datatype "SeachHit" 
 * could contain more then one results and the GetResponse just one
 * </p>
 * 
 * @author Maximilian Brodhun, SUB Goettingen
 * @author Stefan E. Funk, SUB Goettingen
 * @version 24.05.2019
 * @since 18.02.2014
 */
public class DublinCoreFieldLoader {

  private static Log log = LogFactory.getLog(OAIPMHImpl.class);

  /**
   * This function takes the results of the ElasticSearch resuest 
   * and fills the String-Lists with the specific values for the dublin core field
   * @param responseOfGetRequest result of the ElasticSearch request from a getRequest (Single Result)
   * @param fields String List for the specific dublin core field
   * @return String List containing the values for the specific dublin core field
   */
    
  @SuppressWarnings("unchecked")
  public static List<String> fillList(GetResponse responseOfGetRequest, String[] fields){
	  List<String> list = new ArrayList<String>();
	  /**
	   * A stepwise
	   */
	    if (responseOfGetRequest.isExists()) {
	        if (fields != null) {	          
	          for (String field : fields) {    
	            if(responseOfGetRequest.getSourceAsMap().get(field)==null ) {	
		          String[] requestedField = field.split("\\.");
		          Map<String, Object> nestedMap = responseOfGetRequest.getSourceAsMap();
		          String valueOfRequestedField=null;
		          Map<String, Object> nestedMap2 = null;
		          for(int i=0; i<requestedField.length; i++) {
		            if(i<requestedField.length-1 && nestedMap!=null && nestedMap.get(requestedField[i])!=null) {
		              nestedMap2 = (Map<String, Object>)nestedMap.get(requestedField[i]);
		            }            
		            if(i==requestedField.length-1 &&  
		            		nestedMap2!=null && nestedMap2.get(requestedField[i])!=null ) {
		              valueOfRequestedField =nestedMap2.get(requestedField[i]).toString();
		              if(valueOfRequestedField.length()>0) {
		              	list.add(valueOfRequestedField);
		              }
		            }
		            nestedMap= nestedMap2;
		          }
	            }else {
	            	list.add(responseOfGetRequest.getSourceAsMap().get(field).toString());
	            }
	        }
	      }
	    }
	  
	  return list;
  }
  /**
   * This function takes the results of the ElasticSearch resuest 
   * and fills the String-Lists with the specific values for the dublin core field
   * @param hit result of the ElasticSearch request from a multi get Request 
   * @param fields String List for the specific dublin core field
   * @return String List containing the values for the specific dublin core field
   */
  @SuppressWarnings("unchecked")
  public static List<String> fillList(SearchHit hit, String[] fields){
	  List<String> list = new ArrayList<String>();	  
	    
	        if (fields != null) {	          
	          for (String field : fields) {    
	            if(hit.getSourceAsMap().get(field)==null ) {	
		          String[] requestedField = field.split("\\.");
		          Map<String, Object> nestedMap = hit.getSourceAsMap();
		          String valueOfRequestedField=null;
		          Map<String, Object> nestedMap2 = null;
		          for(int i=0; i<requestedField.length; i++) {
		            if(i<requestedField.length-1 && nestedMap!=null && nestedMap.get(requestedField[i])!=null) {
		            	try {
		            		nestedMap2 = (Map<String, Object>)nestedMap.get(requestedField[i]);	
		            	}catch(ClassCastException cce) {
		            		log.info("could not transform the elasticSearch result");
		            	}		              
		            }            
		            if(i==requestedField.length-1 &&  
		            		nestedMap2!=null && nestedMap2.get(requestedField[i])!=null ) {
		              valueOfRequestedField =nestedMap2.get(requestedField[i]).toString();
		              if(valueOfRequestedField.length()>0) {
		              	list.add(valueOfRequestedField);
		              }
		            }
		            nestedMap= nestedMap2;
		          }
	            }else {
	            	list.add(hit.getSourceAsMap().get(field).toString());
	            }
	        }
	      }	  
	  return list;
  }

}
