package info.textgrid.middleware;

import java.io.IOException;

import org.elasticsearch.action.get.GetRequest;
import org.elasticsearch.action.get.GetResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.Strings;
import org.elasticsearch.search.fetch.subphase.FetchSourceContext;

import info.textgrid.middleware.oaipmh.ListMetadataFormatsType;
import info.textgrid.middleware.oaipmh.MetadataFormatType;
import info.textgrid.middleware.oaipmh.RequestType;

public class MetadataFormatListDeliverer {
	
	private boolean idExist = true;
	
	public MetadataFormatListDeliverer(){

	}
		
	public ListMetadataFormatsType setMetadataFormatList(String id){
		
	    String[] includes = new String[]{TGConstants.URI};
	    String[] excludes = Strings.EMPTY_ARRAY;
	    FetchSourceContext fetchSourceContext =
	        new FetchSourceContext(true, includes, excludes);
		
		ListMetadataFormatsType lmft = new ListMetadataFormatsType();
		MetadataFormatType mft = new MetadataFormatType();
		GetRequest getRequest = new GetRequest(OAI_ESClient.getEsIndex(), OAI_ESClient.getEsType(), id).fetchSourceContext(fetchSourceContext);
		
		
		/*
		GetRequestBuilder recordById = OAI_ESClient.getEsClient().get(getRequest, GetAction.INSTANCE)
			.setIndex(OAI_ESClient.getEsIndex())
			.setType(OAI_ESClient.getEsType())
			.setFields(TGConstants.URI)
			.setId(id); */
		
		GetResponse tgObject = null;
		
		try {
			tgObject=OAI_ESClient.getEsClient().get(getRequest, RequestOptions.DEFAULT);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(tgObject.isExists()){
			setIdExist(true);
			mft.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
			mft.setMetadataPrefix(TGConstants.METADATA_DC_PREFIX);
			mft.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");
			lmft.getMetadataFormat().add(mft);
		}else{
			setIdExist(false);
			lmft=null;
		}
				
		return lmft;
	}
	
	public ListMetadataFormatsType setMetadataFormatListWithoutId(){
		
		ListMetadataFormatsType lmft = new ListMetadataFormatsType();
		MetadataFormatType mft = new MetadataFormatType();
		MetadataFormatType mftIdiomMets = new MetadataFormatType();
		
		mft.setMetadataNamespace("http://www.openarchives.org/OAI/2.0/oai_dc/");
		mft.setMetadataPrefix(TGConstants.METADATA_DC_PREFIX);
		mft.setSchema("http://www.openarchives.org/OAI/2.0/oai_dc.xsd");
		lmft.getMetadataFormat().add(mft);
		
		mftIdiomMets.setMetadataNamespace("http://www.loc.gov/METS/");
		mftIdiomMets.setMetadataPrefix(TGConstants.METADATA_IDIOM_PREFIX);
		mftIdiomMets.setSchema("http://www.loc.gov/standards/mets/mets.xsd");
		lmft.getMetadataFormat().add(mftIdiomMets);
		return lmft;
		
	}
	
	public boolean requestChecker(RequestType request){
		boolean requestCheck = true;
		
		if(request.getFrom()!=null ||
		   request.getMetadataPrefix()!=null ||
		   request.getResumptionToken()!=null){
			
			requestCheck = false;
			
		}else if(request.getSet()!=null ||
				 request.getUntil()!=null){
			requestCheck = false;
		}
		
		return requestCheck;
		
	}
	
	public boolean isIdExist() {
		return idExist;
	}

	public void setIdExist(boolean idExist) {
		this.idExist = idExist;
	}
	
}
